@@ -299,7 +299,9 @@ void trimStart_Rangecc(iRangecc *d) {
     while (pos != (const uint8_t *) d->end) {
         iChar ch;
         pos = u8_next(&ch, pos);
-        if (!isSpace_Char(ch)) break;
+        /* Variation selectors follow the main codepoint, so if one is found at the beginning
+           it should be ignored. */
+        if (!isSpace_Char(ch) && !isVariationSelector_Char(ch)) break;
         d->start = (const char *) pos;
     }
 }
@@ -756,11 +758,21 @@ const iString *string_Rangecc(iRangecc range) {
     return collect_String(newRange_String(range));
 }
 
+iLocalDef int cmpNullRange_(const char *cstr) {
+    return (cstr == NULL || *cstr == 0 ? 0 : -1);
+}
+
 int cmpCStrSc_Rangecc(const iRangecc d, const char *cstr, const iStringComparison *sc) {
+    if (isNull_Rangecc(d)) {
+        return cmpNullRange_(cstr);
+    }
     return cmpCStrNSc_Rangecc(d, cstr, strlen(cstr), sc);
 }
 
 int cmpCStrNSc_Rangecc(const iRangecc d, const char *cstr, size_t n, const iStringComparison *sc) {
+    if (isNull_Rangecc(d)) {
+        return cmpNullRange_(cstr);
+    }
     const size_t size = size_Range(&d);
     int cmp = sc->cmpN(d.start, cstr, iMin(n, size));
     if (cmp == 0) {
