@@ -51,6 +51,7 @@ struct Impl_Prefs {
     int              zoomPercent;
     iBool            sideIcon;
     iBool            hideToolbarOnScroll;
+    int              pinSplit; /* 0: no pinning, 1: left doc, 2: right doc */
     /* Behavior */
     iString          downloadDir;
     iBool            hoverLink;
@@ -58,6 +59,7 @@ struct Impl_Prefs {
     iBool            loadImageInsteadOfScrolling;
     iBool            collapsePreOnLoad;
     iString          searchUrl;
+    iBool            openArchiveIndexPages;
     /* Network */
     iString          caFile;
     iString          caPath;
