@@ -32,11 +32,19 @@ enum iInputMode {
     overwrite_InputMode,
 };
 
+iDeclareType(InputWidgetContentPadding)
+
+struct Impl_InputWidgetContentPadding {
+    int left;
+    int right;
+};
+
 void    setHint_InputWidget     (iInputWidget *, const char *hintText);
 void    setMode_InputWidget     (iInputWidget *, enum iInputMode mode);
 void    setMaxLen_InputWidget   (iInputWidget *, size_t maxLen);
 void    setText_InputWidget     (iInputWidget *, const iString *text);
 void    setTextCStr_InputWidget (iInputWidget *, const char *cstr);
+void    setFont_InputWidget     (iInputWidget *, int fontId);
 void    setCursor_InputWidget   (iInputWidget *, size_t pos);
 void    setContentPadding_InputWidget   (iInputWidget *, int left, int right); /* only affects the text entry */
 void    begin_InputWidget       (iInputWidget *);
@@ -49,7 +57,9 @@ void    setUrlContent_InputWidget       (iInputWidget *, iBool isUrl);
 void    setNotifyEdits_InputWidget      (iInputWidget *, iBool notifyEdits);
 void    setEatEscape_InputWidget        (iInputWidget *, iBool eatEscape);
 
-const iString * text_InputWidget    (const iInputWidget *);
+const iString * text_InputWidget            (const iInputWidget *);
+iInputWidgetContentPadding
+                contentPadding_InputWidget  (const iInputWidget *);
 
 iLocalDef iInputWidget *newHint_InputWidget(size_t maxLen, const char *hint) {
     iInputWidget *d = new_InputWidget(maxLen);
