@@ -427,7 +427,7 @@ void submit_LookupWidget(iLookupWidget *d, const iString *term) {
         trim_String(&d->pendingTerm);
         iReleasePtr(&d->pendingDocs);
         if (!isEmpty_String(&d->pendingTerm)) {
-            d->pendingDocs = listDocuments_App(); /* holds reference to all open tabs */
+            d->pendingDocs = listDocuments_App(get_Root()); /* holds reference to all open tabs */
             signal_Condition(&d->jobAvailable);
         }
         else {
@@ -654,20 +654,21 @@ static iBool processEvent_LookupWidget_(iLookupWidget *d, const SDL_Event *ev) {
              (equal_Command(cmd, "layout.changed") &&
               equal_Rangecc(range_Command(cmd, "id"), "navbar"))) {
         /* Position the lookup popup under the URL bar. */ {
-            const iWindow *window = get_Window();
-            const iInt2 rootSize = rootSize_Window(window);
-            const iRect navBarBounds = bounds_Widget(findWidget_App("navbar"));
-            setFixedSize_Widget(w, init_I2(width_Widget(findWidget_App("url")),
-                                           (rootSize.y - bottom_Rect(navBarBounds)) / 2));
-            setPos_Widget(w, bottomLeft_Rect(bounds_Widget(findWidget_App("url"))));
+            iRoot *root = w->root;
+            const iRect navBarBounds = bounds_Widget(findChild_Widget(root->widget, "navbar"));
+            iWidget *url = findChild_Widget(root->widget, "url");
+            setFixedSize_Widget(w, init_I2(width_Widget(url),
+                                           (bottom_Rect(rect_Root(root)) - bottom_Rect(navBarBounds)) / 2));
+            setPos_Widget(w, windowToLocal_Widget(w, bottomLeft_Rect(bounds_Widget(url))));
 #if defined (iPlatformAppleMobile)
             /* Adjust height based on keyboard size. */ {
                 w->rect.size.y = visibleRootSize_Window(window).y - top_Rect(bounds_Widget(w));
                 if (deviceType_App() == phone_AppDeviceType) {
                     float l, r;
                     safeAreaInsets_iOS(&l, NULL, &r, NULL);
-                    w->rect.size.x = rootSize.x - l - r;
+                    w->rect.size.x = size_Root(root).x - l - r;
                     w->rect.pos.x  = l;
+                    /* TODO: Need to use windowToLocal_Widget? */
                 }
             }
 #endif
@@ -695,7 +696,7 @@ static iBool processEvent_LookupWidget_(iLookupWidget *d, const SDL_Event *ev) {
             setText_InputWidget(url, url_DocumentWidget(document_App()));
             showCollapsed_Widget(w, iFalse);
             setCursor_LookupWidget_(d, iInvalidPos);
-            postCommandString_App(&item->command);
+            postCommandString_Root(get_Root(), &item->command);
             postCommand_App("focus.set id:"); /* unfocus */
         }
         return iTrue;
@@ -739,7 +740,8 @@ static iBool processEvent_LookupWidget_(iLookupWidget *d, const SDL_Event *ev) {
                     return iTrue;
             }
         }
-        if (key == SDLK_DOWN && !mods && focus_Widget() == findWidget_App("url") &&
+        if (isVisible_Widget(w) &&
+            key == SDLK_DOWN && !mods && focus_Widget() == findWidget_App("url") &&
             numItems_ListWidget(d->list)) {
             setCursor_LookupWidget_(d, 1); /* item 0 is always the first heading */
             setFocus_Widget(w);
